// PtBaseTuner.h: PT`[iNX
//
/////////////////////////////////////////////////////////////////////////////

#pragma once


#include "PtDevice.h"
#include <vector>
#include <queue>


/////////////////////////////////////////////////////////////////////////////
// PT1`[iNX
/////////////////////////////////////////////////////////////////////////////

class CPtBaseTuner :	public CBonObject,
						public IHalDevice,
						public IHalTsTuner,
						protected CPtTuner::ITsReceiver
{
public:
// CBonObject
	DECLARE_IBONOBJECT(CPtBaseTuner)

// IHalDevice
	virtual const BONGUID GetDeviceType(void);
	virtual const DWORD GetDeviceName(LPTSTR lpszName);
	virtual const DWORD GetTotalDeviceNum(void);
	virtual const DWORD GetActiveDeviceNum(void);

// IHalTsTuner
	virtual const bool OpenTuner(void);
	virtual void CloseTuner(void);
	
	virtual const DWORD EnumTuningSpace(const DWORD dwSpace, LPTSTR lpszSpace);
	virtual const DWORD EnumChannelName(const DWORD dwSpace, const DWORD dwChannel, LPTSTR lpszChannel);
	
	virtual const DWORD GetCurSpace(void);
	virtual const DWORD GetCurChannel(void);

	virtual const bool SetChannel(const DWORD dwSpace, const DWORD dwChannel);
	virtual const bool SetLnbPower(const bool bEnable);
	virtual const float GetSignalLevel(void);

	virtual const bool GetStream(BYTE **ppStream, DWORD *pdwSize, DWORD *pdwRemain);

	virtual void PurgeStream(void);
	virtual const DWORD WaitStream(const DWORD dwTimeOut = 0UL);
	virtual const DWORD GetAvailableNum(void);

// CPtBaseTuner
	struct PT_CH_CONFIG
	{
		const TCHAR szChName[32];		// `l
		const DWORD dwChIndex;			// PT1`lCfbNX
		const WORD wTsID;				// TS ID
	};

	struct PT_CH_SPACE
	{
		const TCHAR szSpaceName[32];	// Ԗ
		const PT_CH_CONFIG *pChConfig;	// `lݒ
		const DWORD dwChNum;			// `l
	};

	CPtBaseTuner(IBonObject *pOwner, const Device::ISDB dwTunerType = Device::ISDB_T, const PT_CH_SPACE *pTuningSpace = NULL, const DWORD dwSpaceNum = 0UL);
	virtual ~CPtBaseTuner(void);

protected:
// CPtTuner::ITsReceiver
	virtual void OnTsStream(CPtTuner *pTuner, const BYTE *pData, const DWORD dwSize);

	static CPtManager *m_pPtManager;
	static DWORD m_dwInstanceCount;
	
	CPtTuner *m_pTuner;
	
	const Device::ISDB m_dwTunerType;
	const PT_CH_SPACE * const m_pTuningSpace;
	const DWORD m_dwSpaceNum;

	DWORD m_dwCurSpace;
	DWORD m_dwCurChannel;
	
	CSmartLock m_FifoLock;
	CSmartEvent m_FifoEvent;

	std::vector<CMediaData> m_BlockPool;
	std::vector<CMediaData>::iterator m_itFreeBlock;
	std::queue<CMediaData *> m_FifoBuffer;
};
